@@ -1,4 +1,4 @@
-# -*- coding: utf-8 -*-
+from .. import logger
 from .storage import *
 from .progress_imp2 import *
 from .queries import *
