@@ -1,10 +1,10 @@
-# -*- coding: utf-8 -*-
 import inspect
-from typing import Dict, Set
+import traceback
+from typing import cast, Dict, List, Set
 
 from zuper_commons.types import check_isinstance, describe_type, describe_value, raise_wrapped
 from .helpers import get_commands, UIState
-from .visualization import warning
+from .visualization import ui_warning
 from .. import CompmakeConstants, get_compmake_config, get_compmake_status
 from ..constants import DefaultsToConfig
 from ..context import Context
@@ -22,7 +22,7 @@ from ..jobs import (
 )
 from ..jobs.actions import clean_cache_relations
 from ..jobs.storage import db_job_add_parent_relation, get_job_args
-from ..structures import Job, Promise, same_computation
+from ..structures import CMJobID, Job, Promise, same_computation
 from ..utils import interpret_strings_like, try_pickling
 
 
@@ -102,8 +102,6 @@ def clean_other_jobs(context):
     # logger.info('Cleaning all jobs not defined in this session.'
     #                ' Previous: %d' % len(defined_now))
 
-    from compmake.ui import info
-
     todelete = set()
 
     for job_id in all_jobs(force_db=True, db=db):
@@ -116,11 +114,14 @@ def clean_other_jobs(context):
 
             who = job.defined_by[1:]
             if who:
-                defined = " (defined by %s)" % "->".join(who)
+                a = "->".join(who)
+                defined = f" (defined by {a})"
             else:
                 defined = ""
 
-            info("Job %r not defined in this session%s; cleaning." % (job_id, defined))
+            from compmake.ui.visualization import ui_info
+
+            ui_info(context, f"Job {job_id!r} not defined in this session {defined}; cleaning.")
             #
             #             if not clean_all:
             #                 # info('Job %s defined-by %s' % (job_id, job.defined_by))
@@ -208,7 +209,7 @@ def comp_(context, command_, *args, **kwargs):
                     "http://stefaanlippens.net/pickleproblem "
                     "and the bug report http://bugs.python.org/issue5509."
                 )
-            warning(msg)
+            ui_warning(context, msg)
             WarningStorage.warned.add(command)
 
     if get_compmake_status() == CompmakeConstants.compmake_status_slave:
@@ -297,7 +298,7 @@ def comp_(context, command_, *args, **kwargs):
                 else:
 
                     for i in range(1000):  # XXX
-                        n = "%s-%d" % (job_id, i)
+                        n = cast(CMJobID, f"{job_id}-{i}")
                         if not job_exists(n, db=db):
                             job_id = n
                             break
@@ -478,9 +479,17 @@ def interpret_commands(commands_str: str, context: Context, cq: CacheQueryDB, se
     for cmd in commands:
         try:
             publish(context, "command-starting", command=cmd)
+            # noinspection PyNoneFunctionAssignment
             retcode = interpret_single_command(cmd, context=context, cq=cq)
+
         except KeyboardInterrupt:
-            publish(context, "command-interrupted", command=cmd, reason="KeyboardInterrupt")
+            publish(
+                context,
+                "command-interrupted",
+                command=cmd,
+                reason="KeyboardInterrupt",
+                traceback=traceback.format_exc(),
+            )
             raise
         except UserError as e:
             publish(context, "command-failed", command=cmd, reason=e)
@@ -491,15 +500,15 @@ def interpret_commands(commands_str: str, context: Context, cq: CacheQueryDB, se
             continue
         else:
             if isinstance(retcode, int):
-                publish(context, "command-failed", command=cmd, reason="Return code %r" % retcode)
-                raise CommandFailed("ret code %s" % retcode)
+                publish(context, "command-failed", command=cmd, reason=f"Return code {retcode!r}")
+                raise CommandFailed(f"ret code {retcode}")
             else:
                 publish(context, "command-failed", command=cmd, reason=retcode)
-                raise CommandFailed("ret code %s" % retcode)
+                raise CommandFailed(f"ret code {retcode}")
 
 
 # @contract(returns="None", commands_line="unicode")
-def interpret_single_command(commands_line: str, context, cq) -> None:
+def interpret_single_command(commands_line: str, context, cq: CacheQueryDB):
     """ Returns None or raises CommandFailed """
     if not isinstance(commands_line, str):
         raise ValueError("Expected a string")
@@ -528,19 +537,20 @@ def interpret_single_command(commands_line: str, context, cq) -> None:
     # look for  key=value pairs
     other = []
     kwargs = {}
-    argspec = inspect.getargspec(function)
 
-    defaults = get_defaults(argspec)
-    args_without_default = get_args_without_defaults(argspec)
+    signature = inspect.signature(function)
+
+    defaults = get_defaults(signature)
+    args_without_default = get_args_without_defaults(signature)
 
     for a in args:
         if a.find("=") > 0:
             k, v = a.split("=")
 
-            if not k in argspec.args:
+            if not k in signature.parameters:
                 msg = (
-                    "You passed the argument %r for command %r, "
-                    "but the only available arguments are %s." % (k, cmd.name, argspec.args)
+                    f"You passed the argument {k!r} for command {cmd.name!r}, "
+                    f"but the only available arguments are {signature.parameters}."
                 )
                 raise UserError(msg)
 
@@ -563,7 +573,7 @@ def interpret_single_command(commands_line: str, context, cq) -> None:
 
     args = other
 
-    function_args = argspec.args
+    function_args = signature.parameters
     # set default values
     for argname, argdefault in defaults.items():
         if not argname in kwargs and isinstance(argdefault, DefaultsToConfig):
@@ -578,7 +588,7 @@ def interpret_single_command(commands_line: str, context, cq) -> None:
 
     if "non_empty_job_list" in function_args:
         if not args:
-            msg = "The command %r requires a non empty list of jobs as " "argument." % command_name
+            msg = f"The command {command_name!r} requires a non empty list of jobs as argument."
             raise UserError(msg)
 
         job_list = parse_job_list(args, context=context, cq=cq)
@@ -615,29 +625,18 @@ def interpret_single_command(commands_line: str, context, cq) -> None:
             cq.invalidate()
 
 
-# @contract(returns=dict)
-def get_defaults(argspec) -> Dict:
+def get_defaults(signature: inspect.Signature) -> Dict[str, object]:
     defaults = {}
-    if argspec.defaults:
-        num_args_with_default = len(argspec.defaults)
-    else:
-        num_args_with_default = 0
+    for k, v in signature.parameters.items():
+        if v.default != inspect.Parameter.empty:
+            defaults[k] = v.default
 
-    num_args = len(argspec.args)
-    num_args_without_default = num_args - num_args_with_default
-    for k in range(num_args_without_default, num_args):
-        argname = argspec.args[k]
-        argdefault = argspec.defaults[k - num_args_without_default]
-        defaults[argname] = argdefault
     return defaults
 
 
-def get_args_without_defaults(argspec):
-    if argspec.defaults:
-        num_args_with_default = len(argspec.defaults)
-    else:
-        num_args_with_default = 0
-    num_args = len(argspec.args)
-    num_args_without_default = num_args - num_args_with_default
-    args_without_default = argspec.args[0:num_args_without_default]
+def get_args_without_defaults(signature: inspect.Signature) -> List[str]:
+    args_without_default = []
+    for k, v in signature.parameters.items():
+        if v.default == inspect.Parameter.empty:
+            args_without_default.append(k)
     return args_without_default
