@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 from .compmake_test import CompmakeTest
 from nose.tools import istest
 
@@ -8,7 +6,7 @@ def job_success(*args, **kwargs):
     pass
 
 
-def job_failure(*args, **kwargs):  # @UnusedVariable
+def job_failure(*args, **kwargs):
     assert False, "asserting false"
 
 
